import { PISYS } from "pi_sys/index";
import { Json } from "pi_tools/type";
import { getLogger } from "pi_tools/logger";
import { IUploadUser } from "../interface/upload";
import { UploadPayParam } from "../interface/pay";
import { IPlatformUpload } from "./interface";


declare var module;
let { logD } = getLogger(module.name);

export class BIUpload implements IPlatformUpload {
    lastUser: IUploadUser;
    ad: string = '';  // 广告通道
    ad_id: string = '';  // 广告素材

    constructor() {
        PISYS.Native.App.getChannelInfo().then((r) => {
            logD("BIUpload getChannelInfo", r);
            if (r) {
                this.ad = r.param?.ad || '';
                this.ad_id = r.param?.ad_id || '';
            }
        });
    }

    login(param: IUploadUser, isCreateRole?: boolean): Json[] {

        this.lastUser = JSON.parse(JSON.stringify(param));

        if (isCreateRole) {  // 注册
            let r: Reg = {
                device: param.device,         // 设备
                channel: param.channel,        // 渠道
                ad: this.ad,             // 广告通道
                ad_id: this.ad_id,          // 广告素材
                server: param.serverID?.toString(),         // 区服
                uuid: param.uuid?.toString(),           // 用户唯一ID
                uid: param.userID?.toString(),            // 游戏用户ID
            };

            return [
                filterAttr(r) as Json
            ];

        } else {  // 登录
            let r: Login = {
                device: param.device,         // 设备
                channel: param.channel,        // 渠道
                ad: this.ad,             // 广告通道
                ad_id: this.ad_id,          // 广告素材
                server: param.serverID?.toString(),         // 区服
                uuid: param.uuid?.toString(),           // 用户唯一ID
                uid: param.userID?.toString(),            // 游戏用户ID
                level: param.roleLevel?.toString(),          // 等级
                vip: param.roleVipLevel?.toString(),            // vip等级
                name: param.userName,           // 用户昵称
                version: param.version,        // app版本
                ip: param.ip,             // 客户端IP
                network: param.network,        // 网络(2G/3G/4G/5G/WIFI) 
            };

            return [
                filterAttr(r) as Json
            ];
        }
    }

    logout(): Json[] {
        let r: Logout = {
            uid: this.lastUser.userID?.toString(),            // 游戏用户ID
        }

        return [
            filterAttr(r) as Json
        ];
    }

    pay(param: UploadPayParam): Json[] {
        let r: Pay = {
            uid: this.lastUser.userID?.toString(),            // 游戏用户ID
            orderid: param.orderID?.toString(),          // 订单号
            channel_pay: param.payment,    // 支付渠道
            fee: param.currencyType,            // 货币
            money: param.amount?.toString(),          // 订单金额
            goods: param.goodsID?.toString(),          // 商品
            level: this.lastUser.roleLevel?.toString(),          // 等级
            vip: this.lastUser.roleVipLevel?.toString(),            // vip等级
        }
        return [
            filterAttr(r) as Json
        ];
    }

    // gi 最终值上报  
    // giSum 累加值上报
    custom(type: 'gi' | 'giSum', value?: Json): Json {
        let typeList = ['gi', 'giSum'];
        if (!typeList.includes(type)) return null;
        if (!this.lastUser) return null;

        let r: GI = {
            uid: this.lastUser.userID.toString(),    // 游戏用户ID
            g_k: value?.key,              // key
            g_v: value?.val,              // value
        }
        return [
            filterAttr(r) as Json
        ];
    }
}

// 注册
export interface Reg {
    device: string,         // 设备
    channel: string,        // 渠道
    ad?: string,             // 广告通道
    ad_id?: string,          // 广告素材
    server: string,         // 区服
    uuid?: string,           // 用户唯一ID
    uid: string,            // 游戏用户ID
}

// 登录
export interface Login {
    device: string,         // 设备
    channel: string,        // 渠道
    ad?: string,             // 广告通道
    ad_id?: string,          // 广告素材
    server: string,         // 区服
    uuid?: string,           // 用户唯一ID
    uid: string,            // 游戏用户ID
    level?: string,          // 等级
    vip?: string,            // vip等级
    name?: string,           // 用户昵称
    version?: string,        // app版本
    ip?: string,             // 客户端IP
    network?: string,        // 网络(2G/3G/4G/5G/WIFI) 
}

// 登出
export interface Logout {
    uid: string,            // 游戏用户ID
}

// 充值
export interface Pay {
    uid: string,            // 游戏用户ID
    orderid: string,          // 订单号
    channel_pay: string,    // 支付渠道
    fee: string,            // 货币
    money: string,          // 订单金额
    goods?: string,          // 商品
    level?: string,          // 等级
    vip?: string,            // vip等级
}

// GI
export interface GI {
    uid: string,            // 游戏用户ID
    g_k: string,              // key
    g_v: string,              // value
}

/**
 * 将json中为空的属性删除
 * @param json 
 */
const filterAttr = (json: Object) => {
    for (const k in json) {
        if (json.hasOwnProperty(k)) {
            if (!json[k]) {
                delete json[k];
            }
        }
    }

    return json;
}