/**
 * 后端bi上报sdk
 */
import { PISYS } from 'pi_sys/index';
import { Logger } from 'pi_sys/util/logger';

declare var module;

let logger = new Logger(module.name, "bi");
let giLogger = new Logger(module.name, "gi");

// TODO 目前只支持前端上报,并且需要再game_env.js中配置对应loggers
let videoLogger = new Logger(module.name, "video_log");

// 注册
export interface Reg {
    device: string,         // 设备
    channel: string,        // 渠道
    ad?: string,             // 广告通道
    ad_id?: string,          // 广告素材
    server: string,         // 区服
    uuid?: string,           // 用户唯一ID
    uid: string,            // 游戏用户ID
}

// 登录
export interface Login {
    device: string,         // 设备
    channel: string,        // 渠道
    ad?: string,             // 广告通道
    ad_id?: string,          // 广告素材
    server: string,         // 区服
    uuid?: string,           // 用户唯一ID
    uid: string,            // 游戏用户ID
    level?: string,          // 等级
    vip?: string,            // vip等级
    name?: string,           // 用户昵称
    version?: string,        // app版本
    ip?: string,             // 客户端IP
    network?: string,        // 网络(2G/3G/4G/5G/WIFI) 
}

// 登出
export interface Logout {
    uid: string,            // 游戏用户ID
}

// 充值
export interface Pay {
    uid: string,            // 游戏用户ID
    orderid: string,          // 订单号
    channel_pay: string,    // 支付渠道
    fee: string,            // 货币
    money: string,          // 订单金额
    goods?: string,          // 商品
    level?: string,          // 等级
    vip?: string,            // vip等级
}

// GI
export interface GI {
    uid: string,            // 游戏用户ID
    g_k: string,              // key
    g_v: string,              // value
}

// ABtest
export interface ABTest {
    uid: string,            // 游戏用户ID
    tagName: string,        // tag.id + "." + tag.tag;
}

// 视频数据上报
export interface Video {
    request_id: string,         // 埋点ID/请求ID
    user_id: string,            // 用户唯一ID
    exp_id?: string,             // 实验唯一ID
    page?: string,              // 页面
    net_type?: string,           // 网络型号
    event_time?: string,        // 行为时间
    item_id: string,            // 内容ID/视频ID
    event: "expr" | "click" | "praise",              // 行为类型  expr(曝光)/click(点击)/praise(点赞)
    playtime: string,           // 播放时长/阅读时长
}

export class BISDK {
    public project: string;
    private isServer: boolean = false;   // 是否是服务端,服务端不受以下字段影响
    private biUpload: boolean = false;  // bi上报
    private biConsole: boolean = false; // bi打印
    private giUpload: boolean = false;  // gi上报
    private giConsole: boolean = false; // gi打印
    constructor(project: string) {
        this.project = project;
        this.biUpload = PISYS.Env.get("biUpload");
        this.biConsole = PISYS.Env.get("biConsole");
        this.giUpload = PISYS.Env.get("giUpload");
        this.giConsole = PISYS.Env.get("giConsole");
        try {
            if ((<any>self)._$cwd) {
                this.isServer = true;
            }
        } catch (e) { }
    }
    // 注册
    register = (arg: Reg[]) => {
        if (!this.isServer && !this.biUpload && !this.biConsole) return; // 既不上报也不打印,直接返回
        let logs = []
        let now = Math.round(Date.now() / 1000);
        arg.forEach(o => {
            logs.push({ bievent: "register", project: this.project, time: now.toString(), ...o });
        });
        const data = {
            __logs__: logs
        }
        logger.info("%o", data);
    }

    // 登录
    login = (arg: Login[]) => {
        if (!this.isServer && !this.biUpload && !this.biConsole) return; // 既不上报也不打印,直接返回
        let logs = []
        let now = Math.round(Date.now() / 1000);
        arg.forEach(o => {
            logs.push({ bievent: "login", project: this.project, time: now.toString(), ...o });
        });
        const data = {
            __logs__: logs
        }
        logger.info("%o", data);
    }

    // 登出
    logout = (arg: Logout[]) => {
        if (!this.isServer && !this.biUpload && !this.biConsole) return; // 既不上报也不打印,直接返回
        let logs = []
        let now = Math.round(Date.now() / 1000);
        arg.forEach(o => {
            logs.push({ bievent: "logout", project: this.project, time: now.toString(), ...o });
        });
        const data = {
            __logs__: logs
        }
        logger.info("%o", data);
    }

    // 充值
    pay = (arg: Pay[]) => {
        if (!this.isServer && !this.biUpload && !this.biConsole) return; // 既不上报也不打印,直接返回
        let logs = []
        let now = Math.round(Date.now() / 1000);
        arg.forEach(o => {
            logs.push({ bievent: "pay", project: this.project, time: now.toString(), ...o });
        });
        const data = {
            __logs__: logs
        }
        logger.info("%o", data);
    }

    // GI数据上报
    gi = (arg: GI[]) => {
        if (!this.isServer && !this.giUpload && !this.giConsole) return; // 既不上报也不打印,直接返回
        let logs = []
        let now = Math.round(Date.now() / 1000);
        arg.forEach(o => {
            o.uid = o.uid.toString();
            o.g_v = o.g_v.toString();
            logs.push({ bievent: "gi", project: this.project, time: now.toString(), ...o });
        });
        const data = {
            __logs__: logs
        }
        giLogger.info("%o", data);
    }
    // GI累加值上报
    giSum = (arg: GI[]) => {
        if (!this.isServer && !this.giUpload && !this.giConsole) return; // 既不上报也不打印,直接返回
        let logs = []
        let now = Math.round(Date.now() / 1000);
        arg.forEach(o => {
            o.uid = o.uid.toString();
            o.g_v = o.g_v.toString();
            logs.push({ bievent: "gisum", project: this.project, time: now.toString(), ...o });
        });
        const data = {
            __logs__: logs
        }
        giLogger.info("%o", data);
    }


    // 自定义数据
    customize = (arg: object) => {
        if (!this.isServer && !this.biUpload && !this.biConsole) return; // 既不上报也不打印,直接返回
        let logs = []
        let now = Math.round(Date.now() / 1000);
        let object: any = { bievent: "customize", project: this.project, time: now.toString() };
        for (const key in arg) {
            object[key] = arg[key].toString();
        }
        logs.push(object);

        const data = {
            __logs__: logs
        }
        logger.info("%o", data);
    }

    // AbTest
    abTest = (arg: ABTest[]) => {
        if (!this.isServer && !this.biUpload && !this.biConsole) return; // 既不上报也不打印,直接返回
        let logs = []
        let now = Math.round(Date.now() / 1000);
        arg.forEach(o => {
            o.uid = o.uid.toString();
            o.tagName = o.tagName.toString();
            logs.push({ bievent: "abtest", project: this.project, time: now.toString(), ...o });
        });
        const data = {
            __logs__: logs
        }
        logger.info("%o", data);
    }

    // Video
    video = (arg: Video[]) => {
        let logs = []
        let now = Math.round(Date.now() / 1000);
        arg.forEach(o => {
            o.user_id = o.user_id.toString();
            o.playtime = o.playtime.toString();
            o.event_time = o.event_time ? o.event_time : now.toString(),
                logs.push({ ...o });
        });
        const data = {
            __logs__: logs
        }
        videoLogger.info("%o", data);
    }
}

