import { PISYS } from "pi_sys/index";
import { IUploadUser,UploadType } from "../interface/upload"
import { UploadPayParam, UploadPayType,} from "../interface/pay"
import { IPlatformUpload } from "./interface";
import { MyUpload } from "./upload_sdks/my";
import { QuickSdkUpload } from "./upload_sdks/quicksdk";
import { TalkingDataUpload } from "./upload_sdks/talkingdata";
import { TapDBUpload } from "./upload_sdks/tapdb";
import { YcUpload } from "./upload_sdks/yc";
import { Json } from "pi_tools/type";
import { DwUpload } from "./upload_sdks/dw";
import { JrttLogUpload } from "./upload_sdks/jrtt_log";
import { BIUpload } from "./bi";
import { BISDK } from "./bi_sdk";
import { OhySdkUpload } from "./upload_sdks/ohy_sdk";
import { MuxueUpload } from "./upload_sdks/muxue";
import { QuickH5Upload } from "./upload_sdks/quick_h5";
import { SanxiangUpload } from "./upload_sdks/sanxiang";
import { QQGameUpload } from "./upload_sdks/qq_game";
import { QuickOverseaUpload } from "./upload_sdks/quick_oversea";
import { QingchengUpload } from "./upload_sdks/qingcheng";
import { DaqianUpload } from "./upload_sdks/daqian";
import { BaiduSdkUpload } from "./upload_ads/baidu";
import { adgameUpload } from "./upload_ads/index";
import { IUploadContext } from "../interface/upload";
declare var QuickSDK;  // quickSDK H5
declare var BRGameSdk; // 弎湘游戏
declare var gameSDK;   // 大千融合 h5

/**
 * 数据上报 是 打包时 有多少个平台，就上报多少个平台；
 */
export class UploadContext implements IUploadContext{

    BISdk: BISDK;               // BI 上报
    sdks: string[];
    lastUserInfo: IUploadUser;  // 上次设置 的 用户信息
    platforms: Map<string, IPlatformUpload>;   // 平台，键=sdk名，值 是 构造函数创建
    other_platforms: Map<string, IPlatformUpload>;  // 其他平台, 键=sdk名，值 是 构造函数创建
    param: IUploadUser //存放在环境变量中的用户信息

    constructor(sdks: string[]) {
        this.sdks = sdks;
        this.platforms = new Map();
        this.other_platforms = new Map();
        for (const sdk of this.sdks) {
            let impl: IPlatformUpload;
            switch (sdk) {
                case UploadType.mengyou:
                    impl = new MyUpload();
                    break;
                case UploadType.quicksdk:
                    impl = new QuickSdkUpload();
                    break;
                case UploadType.ohy_sdk:
                    impl = new OhySdkUpload();
                    break;
                case UploadType.talkingdata:
                    impl = new TalkingDataUpload();
                    break;
                case UploadType.taptap:
                    impl = new TapDBUpload();
                    break;
                case UploadType.yuchen:
                    impl = new YcUpload();
                    break;
                case UploadType.dongwan:
                    impl = new DwUpload();
                    break;
                case UploadType.jrtt_log:
                    impl = new JrttLogUpload();
                    break;
                case UploadType.muxue:
                    impl = new MuxueUpload();
                    break;
                case UploadType.quicksdkio:
                case UploadType.quicksdkios:
                    impl = new QuickOverseaUpload();
                    break;
                case UploadType.daqian:
                    impl = new DaqianUpload();
                    break;
                case UploadType.ks_ad:
                    impl = new QuickSdkUpload();
                    break;
                case UploadType.baidu:
                    impl = new BaiduSdkUpload();
                    break;
                // case UploadType.bi:
                //     // game_env 中配置的项目名
                //     this.BISdk = new BISDK(PISYS.Env.get('name'));
                //     impl = new BIUpload();
                //     break;
                default:
                    // game_env 中配置的项目名 默认都上报bi
                    this.BISdk = new BISDK(PISYS.Env.get('name'));
                    impl = new BIUpload();
                    break;
            }
            if (!impl) {
                const reason = `UploadContext failed, not impl, sdk = ${sdk}`;
                console.error(reason);
                throw new Error(reason);
            }
            this.platforms.set(sdk, impl);
        }
    }

    setPayContext() {
        
    }

    getSdkArray() {
        return [...this.sdks];
    }

    getLastUserInfo() {
        if (this.lastUserInfo) {
            return JSON.parse(JSON.stringify(this.lastUserInfo));
        }
    }

    updateUserInfo(param: IUploadUser) {
        this.lastUserInfo = {
            ...this.lastUserInfo,
            ...param
        }
    }

    async register() {
        await this.setUserParam();

        this.param = this.setDefaultUser(this.param);
        this.lastUserInfo = JSON.parse(JSON.stringify(this.param));
        //用于激活广告的注册
        adgameUpload.registerRole(this.param)

        if (this.param.uploadType === UploadType.quick_h5) {
            const platform = new QuickH5Upload();
            this.other_platforms.set(this.param.uploadType, platform);
            QuickSDK.uploadGameRoleInfo(platform.login(this.param, true)[0], () => {
                console.log('quick h5 upload success');
            });
            return;
        }
        else if (this.param.uploadType === UploadType.sanxiang) {
            const platform = new SanxiangUpload();
            this.other_platforms.set(this.param.uploadType, platform);
            BRGameSdk.gameRole(platform.login(this.param, true)[0]);
            return;
        }
        else if (this.param.uploadType === UploadType.qq_game) {
            const platform = new QQGameUpload();
            this.other_platforms.set(this.param.uploadType, platform);
            platform.upload(platform.login(this.param, true)[0]);
            return;
        }
        else if (this.param.uploadType === UploadType.quicksdkio || this.param.uploadType === UploadType.quicksdkios) {
            const platform = new QQGameUpload();
            this.other_platforms.set(this.param.uploadType, platform);
            PISYS.Native.SpecialCall.specialCall(this.param.uploadType, platform.login(this.param, true)[0]);
            return;
        }
        else if (this.param.uploadType?.startsWith(UploadType.qingcheng)) {
            const platform = new QingchengUpload();

            this.other_platforms.set(this.param.uploadType, platform);
            platform.upload(platform.login(this.param, true)[0], this.param.uploadType);
            return;
        }
        else if (this.param.uploadType === UploadType.ronghe) {
            const platform = new DaqianUpload();
            this.other_platforms.set(this.param.uploadType, platform);
            gameSDK.createRole(platform.login(this.param, true)[0]).then(e => {
                if (e.statusCode === 0) console.log('dqRonghe h5 upload success');
                else console.warn('dqRonghe h5 upload fail')
            });;
            return;
        }

        //这里是处理native层的sdk上报
        for (const [name, platform] of this.platforms) {
            if (name === UploadType.bi) {
                //BI默认全部上报
                this.BISdk.register(platform.login(this.param, true));
                return;
            }
            for (let info of platform.login(this.param, true)) {
                PISYS.Native.Upload.upload(name, info);
            }
        }
    }

    async login() {
        await this.setUserParam();
        this.param = this.setDefaultUser(this.param);
        this.lastUserInfo = JSON.parse(JSON.stringify(this.param));
        //用于激活广告的登录
        adgameUpload.loginRole(this.param)

        if (this.param.uploadType === UploadType.quick_h5) {
            const platform = new QuickH5Upload();
            this.other_platforms.set(this.param.uploadType, platform);
            QuickSDK.uploadGameRoleInfo(platform.login(this.param, false)[0], () => {
                console.log('quick h5 upload success');
            });
            return;
        }
        else if (this.param.uploadType === UploadType.sanxiang) {
            const platform = new SanxiangUpload();
            this.other_platforms.set(this.param.uploadType, platform);
            BRGameSdk.gameRole(platform.login(this.param, false)[0]);
            return;
        }
        else if (this.param.uploadType === UploadType.qq_game) {
            const platform = new QQGameUpload();
            this.other_platforms.set(this.param.uploadType, platform);
            platform.upload(platform.login(this.param, false)[0]);
            return;
        }
        else if (this.param.uploadType === UploadType.quicksdkio || this.param.uploadType === UploadType.quicksdkios) {
            const platform = new QQGameUpload();
            this.other_platforms.set(this.param.uploadType, platform);
            PISYS.Native.SpecialCall.specialCall(this.param.uploadType, 'uploadLogin', platform.login(this.param, false));
            return;
        }
        else if (this.param.uploadType?.startsWith(UploadType.qingcheng)) {
            const platform = new QingchengUpload();
            this.other_platforms.set(this.param.uploadType, platform);
            platform.upload(platform.login(this.param, false)[0], this.param.uploadType);
            return;
        }
        else if (this.param.uploadType === UploadType.ronghe) {
            const platform = new DaqianUpload();
            this.other_platforms.set(this.param.uploadType, platform);
            gameSDK.changeRole(platform.login(this.param, false)[0])
                .then(e => {
                    if (e.statusCode === 0) console.log('dqRonghe h5 upload success');
                    else console.warn('dqRonghe h5 upload fail');
                })
            return;
        }
        else if (this.param.uploadType?.startsWith(UploadType.qingcheng_vivo)) {
            const platform = new QingchengUpload();
            this.other_platforms.set(this.param.uploadType, platform);
            platform.upload(platform.login(this.param, false)[0], this.param.uploadType);
            return;
        }
        for (const [name, platform] of this.platforms) {
            if (name === UploadType.bi) {
                this.BISdk.login(platform.login(this.param, false));
                return;
            }
            for (let info of platform.login(this.param, false)) {
                PISYS.Native.Upload.upload(name, info);
            }
        }
    }

    //激活方法，这里用于除native层的激活调用（例如小游戏，H5等）
    reportActivation() {
        adgameUpload.firstActivation(this.param);
    }

    async logout() {
        await this.setUserParam();
        // 登录成功还未上报
        if (!this.lastUserInfo) return;

        if (this.param.uploadType === UploadType.qq_game) {
            const platform = this.other_platforms.get(this.param.uploadType) as QQGameUpload;
            platform.upload(platform.logout()[0]);
            return;
        }
        else if (this.param.uploadType?.startsWith(UploadType.qingcheng)) {
            const platform = this.other_platforms.get(this.param.uploadType) as QingchengUpload;
            platform.upload(platform.logout()[0], this.param.uploadType);
            return;
        }
        else if (this.param.uploadType === UploadType.ronghe) {
            const platform = this.other_platforms.get(this.param.uploadType);
            gameSDK.changeAccount(platform.logout()[0])
                .then(e => {
                    if (e.statusCode === 0) console.log('dqRonghe h5 upload success');
                    else console.warn('dqRonghe h5 upload fail')
                });
            return;
        }
        for (const [name, platform] of this.platforms) {
            if (name === UploadType.bi) {
                this.BISdk.logout(platform.logout());
                return;
            }
            for (let info of platform.logout()) {
                PISYS.Native.Upload.upload(name, info);
            }
        }
    }

    async pay(payParam:UploadPayParam) {
        await this.setUserParam();
        //native sdk的上报
        for (const [name, platform] of this.platforms) {
            for (let info of platform.pay(payParam)) {
                PISYS.Native.Upload.upload(name, info);
            }
        }
        //用于激活广告的支付
        adgameUpload.pay(this.param, payParam)
    }

    custom(type: string, value?: Json) {
        for (const [name, platform] of this.platforms) {
            let json = platform.custom(type, value);
            if (!json) {
                // 该SDK 不支持 该type
                console.warn(`IUploadContext custom: sdk = ${name} isn't support type = ${type}`);
            } else {
                if (name === UploadType.bi) {
                    this.BISdk[type](json)
                    return;
                }
                return PISYS.Native.Upload.upload(name, json);
            }
        }
    }

    private async setUserParam() {
        let data = PISYS.Env.get('_$sso_user_info') as IUploadUser
        
        this.param = {
            serverID: data.serverID,
            serverName: data.serverName,
            userID : data.userID || "",      // 玩家（角色）ID，纯英文，唯一
            userName : data.userName || "",        // 玩家（角色）名；默认值：""
            userType : data.userType || "",        // 账号类型（来自哪个平台），比如：qq，wx，weibo 等；默认值：""
            userSex : data.userSex || "male",       // 玩家 性别；默认值："unknown"
            userAge : data.userAge || 0,      // 玩家 年龄；默认值：0
            roleLevel : data.roleLevel || 0,      // 角色等级；默认值：0
            roleVipLevel : data.roleVipLevel || 0,        // 用户 vip 充值 等级；默认值：0
            roleBalance : data.roleBalance || "",      // 角色 余额；默认值：""
            partyId : data.partyId || 0,      // 帮派id；默认值：0
            partyName : data.partyName || "",      // 帮派名；默认值：""
            createTime : data.createTime || 0,    // 创建时间；时间戳 默认值：0
            levelUpTime : data.levelUpTime || 0,      // 升级时间；时间戳 默认值：0
    
            device : await PISYS.Native.SystemInfo.getDeviceID(),      // 设备ID
            channel : await PISYS.Native.App.getFlavorName(),      // 渠道
            version : await PISYS.Native.App.getVersionCode() + "",     // APP 版本号
            network :  await PISYS.Native.Network.getType() + "",        // 网络环境
    
            ip : data.ip || "",        // 本机 IP
            uuid : data.uuid || "",         // 用户ID 唯一标识（不分区服）
            fightingCapacity : data.fightingCapacity || "",        // 战力
            uploadType : data.uploadType,
            payTotal : data.payTotal || 0,        //累充
            extend : data.extend || ""         // 扩展串；默认值：""
          };

    }

    private setDefaultUser(info: IUploadUser) {
        info.serverName = info.serverName || "";
        info.userName = info.userName || "";
        info.userType = info.userType || "";
        info.userSex = info.userSex || "unknown";
        info.userAge = info.userAge || 0;
        info.roleLevel = info.roleLevel || 0;
        info.roleVipLevel = info.roleVipLevel || 0;
        info.roleBalance = info.roleBalance || "";
        info.partyName = info.partyName || "";
        info.extend = info.extend || "";
        info.createTime = info.createTime || 0;
        info.levelUpTime = info.levelUpTime || 0;
        info.data_level = info.data_level || 0;
        info.data_total_money = info.data_total_money || 0;
        info.data_blueprint = info.data_blueprint || 0;

        return info;
    }
}
