import { Json } from "pi_tools/type";
import { IPlatformUpload } from "../interface";
import { IUploadUser } from "../../interface/upload";
import {  UploadPayParam } from "../../interface/pay";
export class DaqianUpload implements IPlatformUpload {

    lastUser: IUploadUser;

    login(param: IUploadUser, isCreateRole?: boolean): Json[] {
        this.lastUser = JSON.parse(JSON.stringify(param));
        const r = this.convertParameter(this.lastUser, isCreateRole);
        r.type = isCreateRole ? CustomType.CREATE_ROLE : CustomType.ENTRY_GAME;
        return [
            r as any as Json
        ];
    }

    logout(): Json[] {
        const r = this.convertParameter(this.lastUser);
        r.type = CustomType.EXIT_GAME;
        return [r as any as Json];
    }

    pay(_param: UploadPayParam): Json[] {
        return [];
    }

    custom(type: CustomType, param: Json): Json {
        const data = Object.assign(this.lastUser, {
            ...param,
            type: type,
            userID: this.lastUser.userID,
            userName: this.lastUser.userName,
            serverID: this.lastUser.serverID,
            serverName: this.lastUser.serverName
        }) as IUploadUser;
        const r = this.convertParameter(data);
        return [
            r as any as Json
        ];
    }

    /**
     * 
     * 将 IUploadUser 转换成 平台上报的 参数 
    */
    private convertParameter(param: IUploadUser, isCreateRole = false): IDaQian {
        const others = isCreateRole ? {
            roleCreateTime: Math.floor((param.createTime || 0) / 1000)
        } : {};
        return {
            roleId: param.userID + '',
            roleName: param.userName,
            serverId: param.serverID,
            power: param.power ? param.power + '' : '0',
            task_id: param.task_id || 0,
            serverName: param.serverName,
            payTotal: param.payTotal || 0,
            honor_id: param.honor_id || 0,
            roleLevel: param.roleLevel ? param.roleLevel + '' : '0',
            partyname: param.partyName || '',
            task_name: param.task_name || '',
            vipLevel: param.roleVipLevel ? param.roleVipLevel + '' : '0',
            userMoney: param.roleBalance || '0',
            honor_name: param.honor_name || '',
            profession: param.profession || '',
            task_status: param.task_status || 0,
            partyroleid: param.partyroleid || 0,
            professionId: param.professionId || 0,
            partyrolename: param.partyrolename || '',
            roleCTime: param.createTime + ''.slice(0, 10),
            partyid: param.partyId ? String(param.partyId) : '',
            roleLevelUpTime: Math.floor((param.levelUpTime || 0) / 1000),
            banlance: param.banlance || 0,
            friendList: param.friendList || 0,
            gender: param.userSex === 'male' ? '男' : param.userSex === "female" ? '女' : '',
            ...others
        }
    }
}

// 大千
interface IDaQian {
    type?: string,           //  String 上报类型
    roleId: string,         //	String	Y	角色 id
    roleName: string,	    //  String	Y	角色名
    roleLevel: string,      //	String	Y	角色等级
    serverId: string,       //	String	Y	角色所在服 id （如果合服,传递合服前区服ID）
    serverName: string,     //	String	Y	角色所在服名
    vipLevel: string,       //  String	Y	角色 vip 等级
    userMoney: string,      //	String	Y	角色用户余额
    roleCTime: string,      //	String	Y	角色创建时间（10 位 unix 时间戳）
    gender: string,         //	String	Y	性别（若无传“”，有传男、女）
    professionId: number,   //	Number	Y	当前登录玩家的职业 ID（若无传 0）
    profession: string,     //	String	Y	当前登录玩家的职业名称（若无传“”）
    power: string,          //	String	Y	战力值 （若无传””）
    partyid: string,        //	String	Y	当前玩家的所属帮派帮派 ID（若无传“”）
    partyname: string,      //	String	Y	当前玩家的所属帮派帮派名称（若无传“”）
    partyroleid: number,    //	Number	Y	帮派称号 ID（若无传 0）
    partyrolename: string,  //	String	Y	帮派称号名称（若无传“”）
    friendList?: Object,     //	Object	Y	朋友关系（若无不传）
    banlance?: Object,       //	Object	Y	钱包 (若无不传)
    payTotal: number,           //	Number	Y	累计充值金额 元为单位 (新增字段)
    roleCreateTime?: number,  //	Int	Y	精确到秒 当[创建角色]的时候传,否则不传
    roleLevelUpTime: number, //	Int	Y	精确到秒 （若无传“”） 当[等级提升]的时候传,否则不传
    task_id: number,        //	Number	Y	自定义任务id （若无传“”） 当[结束任务 / 玩家完成(日常)任务] 的时候传,否则不传
    task_name: string,      //	String	Y	任务名 （若无传“”） 当[结束任务 / 玩家完成(日常)任务] 的时候传,否则不传
    task_status: number,    //	Number	Y	完成任务状态 1 成功 2 失败 （若无传“”） 当[结束任务 / 玩家完成(日常)任务] 的时候传,否则不传
    honor_id: number,       //	Number	Y	自定义荣誉id （若无传“”） 当[获得荣誉 / 玩家获得某个荣誉] 的时候传,否则不传
    honor_name: string      //	String	Y	荣誉名称 （若无传“”） 当[获得荣誉 / 玩家获得某个荣誉] 的时候传,否则不传
}

export enum CustomType {
    SELECT_SERVER = '1',  //选择服务器
    CREATE_ROLE = '2',    //创建角色
    ENTRY_GAME = '3',     //角色登录
    LEVEL_UP = '4',       //角色升级
    EXIT_GAME = '5',      //登出
    TASK = '6',           //任务
    HONOR = '7',          //获得荣誉
    VIP_UP = '8'          //vip等级提升
}