import { Json } from "pi_tools/type";
import { IUploadUser } from "../../interface/upload";
import { UploadPayParam } from "../../interface/pay";
import { IPlatformUpload } from "../interface";

export class DwUpload implements IPlatformUpload {
    lastUser: IUploadUser;

    login(param: IUploadUser, isCreateRole?: boolean): Json[] {

        this.lastUser = JSON.parse(JSON.stringify(param));

        let r: IDW = {
            _id: isCreateRole ? 'createRole' : 'enterServer',
            roleId: Number(param.userID),
            roleName: param.userName,
            roleLevel: param.roleLevel,
            zoneId: Number(param.serverID),
            zoneName: param.serverName,
            balance: Number(param.roleBalance),
            vip: param.roleVipLevel,
            partyName: param.partyName,
            createTime: param.createTime,
            levelTime: param.levelUpTime
        };

        return [
            r as any as Json
        ];
    }

    logout(): Json[] {
        return [];
    }

    pay(_param: UploadPayParam): Json[] {
        return [];
    }

    custom(_type: 'levelUp', param: Json): Json {
        let r: IDW = {
            _id: _type,
            roleId: Number(this.lastUser.userID),
            roleName: this.lastUser.userName,
            zoneId: Number(this.lastUser.serverID),
            zoneName: this.lastUser.serverName,
            createTime: this.lastUser.createTime,

            partyName: param?.partyName || this.lastUser.partyName,
            vip: param?.roleVipLevel || this.lastUser.roleVipLevel,
            balance: Number(param?.roleBalance || this.lastUser.roleBalance),

            roleLevel: param?.roleLevel,
            levelTime: param?.levelUpTime || 0
        };

        return r;
    }
}

// 动玩
interface IDW {
    _id: "enterServer" | "levelUp" | "createRole";  // 当前情景。目前支持 enterServer(玩家进入服务器)、levelUp（玩家升级）、createRole（玩家创建角色）；
    roleId: number;  // 当前登录的玩家角色ID。必须为数字，若无，可传入userId；
    roleName: string; // 当前登录的玩家角色名。不能为空，不能为null，若无，传入”游戏名称+userId”；
    roleLevel: number; // 当前登录的玩家角色等级。必须为数字，且不能为0，若无，传入1；
    zoneId: number;   // 当前登录的游戏区服ID。必须为数字，若无，传入0；
    zoneName: string; //当前登录的游戏区服名称。不能为空，不能为null，若无，传入"无区服"
    balance: number;   // 当前用户游戏币余额；必须为数字，若无，传入0；
    vip: number;       // 当前用户VIP等级。必须为数字，若无，传入 0；
    partyName: string;   // 当前用户所属帮派。不能为空，不能为null，若无，传入"无帮派"
    createTime: number;   // 角色创建时间（时间戳10位：秒，不可变）。不能为空，不能为null；
    levelTime: number;    // 角色升级时间（时间戳10位：秒）。不能为空，不能为null,若无，传入0；
}
