import { Json } from "pi_tools/type";
import { IUploadUser } from "../../interface/upload";
import { UploadPayParam } from "../../interface/pay";
import { IPlatformUpload } from "../interface";

export class JrttLogUpload implements IPlatformUpload {
    login(param: IUploadUser, isCreateRole?: boolean): Json[] {
        // 非注册 不用再上报
        if (!isCreateRole) return [];

        let r: IJrttLogRegister = {
            upload_type: "register",
            register_type: param.userType,
            register_result: true
        };

        return [
            r as any as Json
        ];
    }

    logout(): Json[] {
        return [];
    }

    pay(param: UploadPayParam): Json[] {
        let r: IJrttLogPay = {
            upload_type: 'pay', // 上报类型 pay | register
            pay_type: param.goodsName,   // 商品类型
            pay_name: param.goodsName,   // 商品名称
            pay_id: param.goodsID,     // 商品ID
            pay_num: param.goodsCount,    // 商品数量 - Int
            pay_channel: param.payment,  //支付渠道
            pay_rmb: param.currencyType,    // 币种
            pay_result: true,  // 是否成功(必传)
            pay_count: param.amount   // 金额(必传) - Int
        }
        return [
            r as any as Json
        ];
    }

    custom(type: string, value?: Json): Json {
        let r = null;

        if (value) {
            switch (type) {
                case "play_session": {
                    let p: IPlaySession = {
                        upload_type: "play_session",
                        current_duration: value["current_duration"],
                        send_times: value["send_times"],
                        session_no: value["session_no"],
                        session_start_time: value["session_start_time"]
                    };
                    r = p;
                }

                    break;
                case "gt_levelup": {
                    let p: IGtLevelup = {
                        upload_type: "gt_levelup",
                        lev: value["lev"],
                        get_exp: value["get_exp"],
                        method: value["method"],
                        aflev: value["aflev"]
                    };
                    r = p;
                }

                    break;
                case "purchase": {
                    let p: IPurchase = {
                        upload_type: "purchase",
                        is_success: value["is_success"],
                        content_type: value["content_type"],
                        content_name: value["content_name"],
                        content_id: value["content_id"],
                        content_num: value["content_num"],
                        payment_channel: value["payment_channel"],
                        currency: value["currency"],
                        currency_amount: value["currency_amount"]
                    };
                    r = p;
                }

                    break;
                case "register_event": {
                    let p: IRegisterEvent = {
                        upload_type: "register_event",
                        device_no: value["device_no"],
                        is_success: value["is_success"],
                        method: value["method"],
                        nick_name: value["nick_name"],
                        register: value["register"],
                        user_id: value["user_id"]
                    };
                    r = p;
                }

                    break;
                case "user_login": {
                    let p: IUserLogin = {
                        upload_type: "user_login",
                        device_no: value["device_no"],
                        nick_name: value["nick_name"],
                        time: value["time"],
                        user_id: value["user_id"]
                    };
                    r = p;
                }

                    break;
                case "gt_init_info": {
                    let p: IGtInitInfo = {
                        upload_type: "gt_init_info",
                        lev: value["lev"],
                        coin_type: value["coin_type"],
                        coin_left: value["coin_left"]
                    };
                    r = p;
                }

                    break;
                default:
                    break;
            }
        }
        return r as any as Json;
    }
}

/**
 * 今日头条上报 支付
 */
export interface IJrttLogPay {
    upload_type: string; // 上报类型 pay | register
    pay_type: string;   // 商品类型
    pay_name: string;   // 商品名称
    pay_id: string;     // 商品ID
    pay_num: number;    // 商品数量 - Int
    pay_channel: string;  //支付渠道
    pay_rmb: string;    // 币种
    pay_result: boolean;  // 是否成功(必传) 成功才会调用，所以一定是true
    pay_count: number;   // 金额(必传) - Int
}

/**
 * 今日头条上报 注册
 */
export interface IJrttLogRegister {
    upload_type: string; // 上报类型 pay | register
    register_type: string;   // 注册方式 wechat | qq | weibo
    register_result: boolean;   // 注册结果，成功才会调用，所以一定是true
}


/**
 * 今日头条上报 游戏时长
 */
export interface IPlaySession {
    upload_type: "play_session";
    current_duration: number;     // 本次上报的使用时长 - Int
    send_times: number;           // - Int
    session_no: number;           // 当天第几次打开游戏 - Int
    session_start_time: number;   // 该session启动的开始时间 - Int
}


/**
 * 今日头条上报 等级升级
 */
export interface IGtLevelup {
    upload_type: "gt_levelup";
    lev: number;         // 当前玩家等级 - Int
    get_exp: number;     // 获得经验 - Int
    method: string;      // 获得经验途径：闯关成功、引导完成、领取奖励等，使用汉字或者英文进行标识
    aflev: number;       // 用户获得经验后等级，如获得经验未导致升级，则lev=aflev，如导致升级，则lev<aflev触发条件：用户获得经验或者等级发生变化时。 - Int
}


/**
 * 今日头条上报 支付2
 */
export interface IPurchase {
    upload_type: "purchase";
    is_success: boolean;            // 只有true才会上报 - 是否成功
    content_type: string;           // 商品类型
    content_name: string;           // 商品名
    content_id: string;             // 商品ID
    content_num: number;            // 商品数量 - Int
    payment_channel: string;        // 支付渠道名
    currency: string;               // 真实货币类型
    currency_amount: number;        // 支付真实货币金额 - Int
}


/**
 * 今日头条上报 注册2
 */
export interface IRegisterEvent {
    upload_type: "register_event";
    device_no: string;              // 
    is_success: boolean;            // 是否成功
    method: string;                 // 登录/注册方式
    nick_name: string;              // 
    register: string;               // 
    user_id: string;                // 
}


/**
 * 今日头条上报 用户登录
 */
export interface IUserLogin {
    upload_type: "user_login";
    device_no: string;              // 
    nick_name: string;              // 
    time: string;                   // 
    user_id: string;                // 
}

/**
 * 今日头条上报 初始化信息(触发条件：用户启动游戏，初始化完成时上报)
 */
export interface IGtInitInfo {
    upload_type: "gt_init_info";
    lev: number;                    //  玩家等级 - Int
    coin_type: string;              //  获得货币的类型
    coin_left: string;              // 用户身上剩余的货币数量
}