import { Json } from "pi_tools/type";
import { IUploadUser } from "../../interface/upload";
import { UploadPayParam } from "../../interface/pay";
import { IPlatformUpload } from "../interface";

export class OhySdkUpload implements IPlatformUpload {
    lastUser: IUploadUser;

    login(param: IUploadUser, isCreateRole?: boolean): Json[] {
        this.lastUser = param;

        let r: IOhySdk = {
             // 头部数据
            channel: param.channel || "",
            serverID: param.serverID || "",
            userID: param.userID || "",
            userType: param.userType || "",
            gameRoleID: param.userID || "", 
            gameUserLevel:  param.roleVipLevel?.toString() || "",
            
            isCreateRole: isCreateRole,
        };

        return [
            r as any as Json,
        ]
    }
    
    logout(): Json[] {
        return [];
    }

    pay(param: UploadPayParam): Json[] {
        let r: IOhySdk = {
            // 头部数据
            channel: this.lastUser?.channel || "",
            serverID: this.lastUser?.serverID || "",
            userID: this.lastUser?.userID || "",
            userType: this.lastUser?.userType || "",
            gameRoleID: this.lastUser?.userID || "", 
            gameUserLevel:  this.lastUser?.roleVipLevel?.toString() || "",
           
            goodsName: param.goodsName || "",
            goodsID: param.goodsID || "",
            amount: param.amount || 0,
        };

       return [
           r as any as Json,
       ]
    }

    custom(_type: string, _value?: Json): Json {
        return null;
    }
}

interface IOhySdk {
    // 头部数据
    channel: string,    // 市场渠道，泛指：安卓买量过程中打N个包分别投放，ios只有一个包
    serverID: string,  // 服务器id，泛指：需要分服游戏产品，如果用户均在同一个服务器，非必须
    userID: string,    // 账号id，泛指：sdk分配的用户id，一个用户id可玩多个游戏，相当于ohayoo通行证账号，接登录sdk打，非必须
    userType: string,  // 用户类型，泛指：正式服用户该参数打0，测试服用户该参数打1，主因测试服调资源会扰乱正常数据流
    gameRoleID: string, // 角色id，泛指：产品内涉及可创建多个角色，比如魔兽世界内术士、法师等。单角色该id唯一，非必须
    gameUserLevel: string, // 用户等级，泛指：游戏内主账号或角色id，无等级体系可通过用户的最高关卡id进行替代

    // 内置事件：是否创建角色
    isCreateRole?: boolean, // 是否 创建角色

    // 内置事件：内购物品
    goodsName?: string,     // 道具名称
    goodsID?: string,       // 道具ID
    amount?: number,        // 总金额，单位：分
}