import { Json } from "pi_tools/type";
import { IUploadUser,UploadType } from "../../interface/upload";
import { UploadPayParam } from "../../interface/pay";
import { IPlatformUpload } from "../interface";
import { PISYS } from "pi_sys/index";
import { httpGet } from "../../base/utils";

declare var qcfxAPI; // 倾程
declare var VIVO_SDK; // 倾城 vivo
declare var ALIPAY_SDK; // 倾程 支付宝
declare var MT_SDK; // 倾程 美团

export class QingchengUpload implements IPlatformUpload {
    lastUser: IQCSDK;

    login(param: IUploadUser, isCreateRole?: boolean): Json[] {
        const Sso = PISYS.Env.get('sso');
        const { appid, token } = PISYS.Env.get("_$pi_sdk_login_result");
        let r: IQCSDK = {
            upload_type: isCreateRole ? "CREATE_ROLE" : "JOIN_SERVER",
            app_id: appid,
            user_id: Sso.user,
            token: token,
            cp_server_id: param.serverID,
            cp_server_name: param.serverName,
            cp_role_id: param.userID,
            cp_role_name: param.userName,
            cp_role_level: param.roleLevel + '',
            cp_role_balance: param.roleBalance,
            cp_role_vip: param.roleVipLevel + '',
            cp_role_party: param.partyName,
            cp_reincarnation: param.reincarnation || '',
            cp_fighting_capacity: param.fightingCapacity || '',
            cp_profession: param.profession || '',
            cp_sex: param.userSex === 'unknown' ? 1 : (param.userSex === 'male' ? 2 : 3),
            request_time: Date.now(),
        };
        this.lastUser = r;
        return [
            r as any as Json,
        ]
    }

    logout(): Json[] {
        let r = this.lastUser;
        r.upload_type = 'EXIT';
        r.request_time = Date.now();

        return [r];
    }

    pay(_param: UploadPayParam): Json[] {
        return [];
    }

    // 上报数据 单点登录服务器上去签名
    async upload(param: IQCSDK, upload_type: UploadType) {
        const sso = PISYS.Env.get('sso')
        let url = sso.url + '/oauth/qcupload';
        let r = <string>await httpGet(url, {}, param, '');
        console.log('upload return r = ', r);
        const rjson = JSON.parse(r);
        if (rjson.return_code !== '0') return;
        // param.qcfx_sign = rjson.msg;
        const uploadData = { ...param, qcfx_sign: rjson.msg }
        if (upload_type === UploadType.qingcheng) {
            qcfxAPI.sdkRole((e) => {
                console.log('qingcheng upload success', e);
            }, uploadData);
        }
        else if (upload_type === UploadType.qingcheng_vivo) {
            VIVO_SDK.role(function (e) {
                console.log('qingcheng vivo upload success', e);
            }, uploadData);
        }
        else if (upload_type === UploadType.qingcheng_zfb) {
            ALIPAY_SDK.role(function (e) {
                console.log('qingcheng zfb upload success', e);
            }, uploadData);
        }
        else if (upload_type === UploadType.qingcheng_mt) {
            MT_SDK.role(function (e) {
                console.log('qingcheng mt upload success', e);
            }, uploadData);
        }
    }

    custom(type: string, value: IQCSDK): Json {
        let r = this.lastUser;
        r.upload_type = type;
        r.request_time = Date.now();
        r.cp_role_name = value.cp_role_name || this.lastUser.cp_role_name;
        r.cp_role_level = value.cp_role_level || this.lastUser.cp_role_level;
        r.cp_role_balance = value.cp_role_balance || this.lastUser.cp_role_balance;
        r.cp_role_vip = value.cp_role_vip || this.lastUser.cp_role_vip;
        r.cp_role_party = value.cp_role_party || this.lastUser.cp_role_party
        r.cp_reincarnation = value.cp_reincarnation || this.lastUser.cp_reincarnation;
        r.cp_fighting_capacity = value.cp_fighting_capacity || this.lastUser.cp_fighting_capacity;
        r.cp_profession = value.cp_profession || this.lastUser.cp_profession;
        return r;
    }
}

interface IQCSDK {
    /**
     * 上报类型：
        UNDEFINED=未定义，
        JOIN_SERVER=进入游戏，
        CREATE_ROLE=创建角色，
        UPGRADE=角色升级，
        FACTION=加入工会，
        EXIT=退出游戏，
        OTHER=其他情况
     */
    upload_type: string,

    app_id: string, // 平台分配的参数
    user_id: string, // 平台唯一用户标识
    token: string, // 平台用户token
    cp_server_id: string, // CP区服id
    cp_server_name: string, // CP区服名称
    cp_role_id: string, // CP角色id
    cp_role_name: string, // CP角色名称
    cp_role_level: string, // CP角色等级
    cp_role_balance: string, // 账户余额
    cp_role_vip: string, // vip等级
    cp_role_party: string, // 帮派名称
    cp_reincarnation: string, // 转生
    cp_fighting_capacity: string, // 战斗力
    cp_profession: string, // 职业
    cp_sex: number, // 性别：1=未知，2=男，3=女
    request_time: number, // 当前时间戳（单位：毫秒）
    qcfx_sign?: string, // 签名串，具体见签名规则,使用server_key进行加密
}