import { PISYS } from "pi_sys/index";
import { Json } from "pi_tools/type";
import { IUploadUser } from "../../interface/upload";
import { UploadPayParam } from "../../interface/pay";
import { IPlatformUpload } from "../interface";
import { httpGet } from "../../base/utils";

export class QQGameUpload implements IPlatformUpload {
    lastUser: IQQZone;

    login(param: IUploadUser, isCreateRole?: boolean): Json[] {
        let data = PISYS.Env.get('_$pi_sdk_login_result');
        if (!data) return [];

        let r: IQQZone = {
            appid: data.appid,
            userip: 0,
            svrip: 0,
            time: Math.floor(Date.now() / 1000),
            domain: 10,
            iworldid: Number(param.serverID) || 1,
            optype: 3,
            actionid: isCreateRole ? 2 : 1,  // 1登录 2注册
            opuid: param.userID,
            opopenid: data.openid,
            level: param.roleLevel,
            dtEventTime: parseTime()
        };
        this.lastUser = r;

        return [
            r as any as Json,
        ]
    }

    logout(): Json[] {
        let r = this.lastUser;
        let now = Math.floor(Date.now());
        r.onlinetime = now - r.time;
        r.time = now;
        r.actionid = 9;
        r.dtEventTime = parseTime()

        return [
            r as any as Json
        ];
    }

    pay(_param: UploadPayParam): Json[] {
        return [];
    }

    // 上报数据 单点登录服务器上去中转
    async upload(param: IQQZone) {
        const sso = PISYS.Env.get('sso')
        let url = sso.url + '/oauth/qqupload';
        let r = <string>await httpGet(url, {}, param, '');
        console.log('upload return r = ', r);

        return r === 'SUCCESS';
    }

    custom(type: string, _value?: Json): Json {
        if (isNaN(Number(type))) return null;
        let r = this.lastUser;
        let now = Math.floor(Date.now());
        r.onlinetime = now - r.time;
        r.time = now;
        r.actionid = Number(type);
        r.dtEventTime = parseTime()

        return r;
    }
}

interface IQQZone {
    appid: number;       // 游戏唯一标识符，由平台提供
    userip: number;       // 用户机器的ip地址，请使用主机字节序
    svrip: number;          // 当前处理用户请求的机器(cgi或者是server)IP, 请使用主机字节序
    time: number;        // 当前用户的操作时间，精确到秒
    domain: number;          // 用于区分从哪个业务平台进入应用： Qzone为1； QQGame为10；
    iworldid: number;      // 非多区多服应用，这里填1。多区多服应用，这里填大区服务器ID
    optype: number;   // 操作类型 支付类操作为1； 留言发表类为2； 写操作类为3； 读操作类为4； 其它为5
    actionid: number; // 操作ID 其中： 登录为1；主动注册为2；用户登出为9；创建角色为12；
    opuid: string;    // 操作者的UID
    opopenid: string;     // 操作者的OpenID
    level: number;  // 操作者的等级
    dtEventTime: string; // 格式为"YYYY-MM-DD hh:mm:ss"
    onlinetime?: number;  // 登录的在线时长
}

const parseTime = () => {
    let date = new Date();
    return `${date.getFullYear()}-${date.getMonth() + 1}-${date.getDate()} ${date.getHours()}:${date.getMinutes()}:${date.getSeconds()}`
}