import { PISYS } from "pi_sys/index";
import { Json } from "pi_tools/type";
import { MD5 } from "pi_tools/md5";
import { IUploadUser } from "../../interface/upload";
import { UploadPayParam } from "../../interface/pay";
import { IPlatformUpload } from "../interface";

export class QuickH5Upload implements IPlatformUpload {

    login(param: IUploadUser, isCreateRole?: boolean): Json[] {
        let data = PISYS.Env.get('_$pi_quick_data');
        if (!data) return [];


        let r: IQSDK = {
            isCreateRole: isCreateRole,
            uid: data.uid,
            username: data.username,
            roleCreateTime: 0,
            serverId: Number(param.serverID),
            serverName: param.serverName,
            userRoleId: param.userID,
            userRoleName: param.userName,
            userRoleBalance: Number(param.roleBalance),
            vipLevel: param.roleVipLevel,
            userRoleLevel: param.roleLevel,
            partyId: param.partyId,
            partyName: param.partyName,

            gameRoleGender: param.userSex,
            gameRolePower: Number(param.fightingCapacity),
            partyRoleId: Number(param.userID),
            partyRoleName: param.userName,
            professionId: "",
            profession: "",
            friendlist: "",
            channel_role_sign: param.channel_role_sign
        };
        // const content = `uid=${r.uid}&username=${r.username}&serverId=${r.serverId}&serverName=${r.serverName}&userRoleId=${r.userRoleId}&userRoleName=${r.userRoleName}`+
        // `&userRoleLevel=${r.userRoleLevel}&vipLevel=${r.vipLevel}&gameRolePower=${r.gameRolePower}&key=${param.key}`; 
        // r.channel_role_sign = MD5.str_md5(content)

        return [
            JSON.stringify(r) as any as Json,
        ]
    }

    logout(): Json[] {
        return [];
    }

    pay(_param: UploadPayParam): Json[] {
        return [];
    }

    custom(_type: string, _value?: Json): Json {
        return null;
    }
}

interface IQSDK {
    isCreateRole: boolean;     // (必传,true创建角色,false进入游戏及角色升级)
    uid: number;            // 渠道UID
    username: string;       // 渠道Username
    roleCreateTime: number;  // 角色创建时间 单位秒
    serverId: number;          // 服务器ID(必传,数字字符串，不能含有中文字符).
    serverName: string;        // 服务器名称(必传).
    userRoleId: string;       // 游戏角色ID(必传).
    userRoleName: string;     // 游戏角色名称(必传).
    userRoleBalance: number;       // 游戏角色用户余额(必传).
    vipLevel: number;          // 设置当前用户vip等级，必须为数字整型字符串,请勿传"vip1"等类似字符串(必传)，例子 "4"
    userRoleLevel: number;    // 设置游戏用户等级(必传)
    partyId: number,           // 设置帮派id，必须为整型字符串(必传)
    partyName: string;         // 设置帮派名称(必传).

    gameRoleGender: string,   // 角色性别 例子："男"
    gameRolePower: number,    // 设置角色战力，必须为整型字符串，例子："38"
    partyRoleId: number,      // 设置角色在帮派中的id，例子："11"
    partyRoleName: string,    // 设置角色在帮派中的名称，例子："帮主"
    professionId: string,      // 设置角色职业id，必须为整型字符串，例子："38"
    profession: string,         // 设置角色职业名称，例子："法师"
    friendlist: string,        // 设置好友关系列表，例子："无"
    channel_role_sign: string, // 渠道角色签名
}