import { Json } from "pi_tools/type";
import { IUploadUser } from "../../interface/upload";
import { UploadPayParam } from "../../interface/pay";
import { IPlatformUpload } from "../interface";
import { PISYS } from "pi_sys/index";

export class QuickOverseaUpload implements IPlatformUpload {

    login(param: IUploadUser, isCreateRole?: boolean): Json[] {
        let r: IQSDK = {
            is_create_role: isCreateRole ? "true" : "false",

            uid: PISYS.Env.get('quick_oversea_uid'),
            server_id: param.serverID,
            server_name: param.serverName,

            game_role_id: param.userID,
            game_role_name: param.userName,

            game_balance: param.roleBalance,
            vip_level: param.roleVipLevel.toString(),

            game_user_level: param.roleLevel.toString(),
        };

        return [
            r as any as Json,
        ]
    }

    logout(): Json[] {
        return [];
    }

    pay(param: UploadPayParam): Json[] {
        if (!param.eventToken) return [];
        let r = {
            eventToken: param.eventToken,
            orderId: param.orderID,
            amount: param.amount,
            currency: param.currencyType
        }
        return [
            r
        ];
    }

    custom(type: string, _value?: Json): Json {
        return {
            eventToken: type,
        };
    }
}

interface IQSDK {
    is_create_role: string;     // (必传,true创建角色,false进入游戏及角色升级)
    uid: string,                // 渠道的用户id
    server_id: string;          // 服务器ID(必传,数字字符串，不能含有中文字符).
    server_name: string;        // 服务器名称(必传).
    game_role_name: string;     // 游戏角色名称(必传).
    game_role_id: string;       // 游戏角色ID(必传).
    game_balance: string;       // 游戏角色用户余额(必传).
    vip_level: string;          // 设置当前用户vip等级，必须为数字整型字符串,请勿传"vip1"等类似字符串(必传)，例子 "4"
    game_user_level: string;    // 设置游戏用户等级(必传)
}