import { Json } from "pi_tools/type";
import { IUploadUser } from "../../interface/upload";
import { UploadPayParam } from "../../interface/pay";
import { IPlatformUpload } from "../interface";

export class QuickSdkUpload implements IPlatformUpload {

    login(param: IUploadUser, isCreateRole?: boolean): Json[] {
        let r: IQSDK = {
            is_create_role: isCreateRole ? "true" : "false",

            server_id: param.serverID,
            server_name: param.serverName,

            game_role_id: param.userID,
            game_role_name: param.userName,

            game_balance: param.roleBalance,
            vip_level: param.roleVipLevel.toString(),

            game_user_level: param.roleLevel.toString(),
            party_name: param.partyName,
            
            role_create_time: param.createTime.toString(),
            party_id: param.partyName || "",
            game_role_gender: param.userSex,
            game_role_power: "",
            party_role_id: "",
            party_role_name: "",
            profession_id: "",
            profession: "",
            friend_list: "",

            data_level: param.data_level.toString(),
            data_total_money: param.data_total_money.toString(),
            data_blueprint: param.data_blueprint.toString(),
        };

        return [
            r as any as Json,
        ]
    }
    
    logout(): Json[] {
        return [];
    }

    pay(_param: UploadPayParam): Json[] {
        return [];
    }

    custom(_type: string, _value?: Json): Json {
        return null;
    }
}

interface IQSDK {
    is_create_role: string;     // (必传,true创建角色,false进入游戏及角色升级)
    server_id: string;          // 服务器ID(必传,数字字符串，不能含有中文字符).
    server_name: string;        // 服务器名称(必传).
    game_role_name: string;     // 游戏角色名称(必传).
    game_role_id: string;       // 游戏角色ID(必传).
    game_balance: string;       // 游戏角色用户余额(必传).
    vip_level: string;          // 设置当前用户vip等级，必须为数字整型字符串,请勿传"vip1"等类似字符串(必传)，例子 "4"
    game_user_level: string;    // 设置游戏用户等级(必传)
    party_name: string;         // 设置帮派名称(必传).

    role_create_time: string;   // UC，当乐与1881，TT渠道必传，值为10位数时间戳，例子："1473141432"
    party_id: string,           // 360渠道参数，设置帮派id，必须为整型字符串，例子："1100"
    game_role_gender: string,   // 360渠道参数，例子："男"
    game_role_power: string,    // 360,TT语音渠道参数，设置角色战力，必须为整型字符串，例子："38"
    party_role_id: string,      // 360渠道参数，设置角色在帮派中的id，例子："11"
    party_role_name: string,    // 360渠道参数，设置角色在帮派中的名称，例子："帮主"
    profession_id: string,      // 360渠道参数，设置角色职业id，必须为整型字符串，例子："38"
    profession: string,         // 360渠道参数，设置角色职业名称，例子："法师"
    friend_list: string,        // 360渠道参数，设置好友关系列表，例子："无"

    // 咪咕 新增
    data_level: string,       // 玩家等级
    data_total_money: string, // 商铺累计收益
    data_blueprint: string,   // 装备图鉴数量
}