import { PISYS } from "pi_sys/index";
import { Json } from "pi_tools/type";
import { IUploadUser } from "../../interface/upload";
import { UploadPayParam } from "../../interface/pay";
import { IPlatformUpload } from "../interface";

export class SanxiangUpload implements IPlatformUpload {
    lastUser: ISanxiang;

    login(param: IUploadUser, isCreateRole?: boolean): Json[] {
        let data = PISYS.Env.get('_$pi_sdk_login_result');
        if (!data) return [];

        let sex = 0;
        if (param.userSex === 'male') sex = 1;
        if (param.userSex === 'female') sex = 2;
        let r: ISanxiang = {
            uploadType: isCreateRole ? 3 : 2,
            appId: data.appId,
            userId: data.userId,
            serverId: param.serverID,
            serverName: param.serverName,
            roleId: param.userID,
            roleName: param.userName,
            balance: Number(param.roleBalance),
            roleVip: param.roleVipLevel.toString(),
            roleLevel: param.roleLevel.toString(),
            partyName: param.partyName,
            fightingCapacity: param.fightingCapacity || '',
            reincarnation: param.reincarnation || '',
            profession: param.profession || '',
            sex,
        };
        this.lastUser = r;

        return [
            r as any as Json,
        ]
    }

    logout(): Json[] {
        let r = this.lastUser;
        r.uploadType = 6;

        return [
            r as any as Json
        ];
    }

    pay(_param: UploadPayParam): Json[] {
        return [];
    }

    custom(_type: string, _value?: Json): Json {
        return null;
    }
}

interface ISanxiang {
    appId: string;       // 游戏唯一标识符，由平台提供
    userId: string;       // 游戏平台玩家账号唯一标志 userId
    serverId: string;          // 游戏区服 ID
    serverName: string;        // 游戏区服名称
    roleId: string;            // 游戏玩家角色 ID
    roleName: string;      // 游戏玩家角色名称
    roleLevel: string;    // 游戏用户等级
    balance: number;     // 游戏玩家角色余额（默认 0）
    roleVip: string;    // 游戏玩家角色 VIP 等级
    partyName: string;        // 玩家加入公会的名称（默认“无帮派”）
    fightingCapacity: string;      // 战力
    reincarnation: string;          // 转生
    profession: string;     // 角色在游戏内的职业
    sex: number;     // 性别[0=未知,1=男,2=女]
    uploadType: number;     // 上报类型[1=未定义，2=进入游戏，3=创建角色，4=角色升级，5=加入帮派，6=退出游戏，7=其他情况]
}