import { Json } from "pi_tools/type";
import { IUploadUser } from "../../interface/upload";
import { UploadPayParam } from "../../interface/pay";
import { IPlatformUpload } from "../interface";

export class TapDBUpload implements IPlatformUpload {

    lastUser: IUploadUser;

    login(param: IUploadUser, _isCreateRole?: boolean): Json[] {
        let arr = [];

        if (!this.lastUser) {
            this.lastUser = this.createDefaultUser();
        }

        if (this.lastUser.userID !== param.userID) {
            arr.push({
                upload_type: "user",
                user: param.userID,
            });
        }

        if (this.lastUser.userName !== param.userName) {
            arr.push({
                upload_type: "name",
                name: param.userName,
            });
        }

        if (this.lastUser.roleLevel !== param.roleLevel) {
            arr.push({
                upload_type: "level",
                level: param.roleLevel,
            });
        }

        if (this.lastUser.serverID !== param.serverID
            || this.lastUser.serverName !== param.serverName
        ) {
            let server = param.serverID;
            if (param.serverName) {
                server += ":" + param.serverName;
            }
            arr.push({
                upload_type: "server",
                server,
            });
        }

        this.lastUser = JSON.parse(JSON.stringify(param));
        return arr;
    }

    logout(): Json[] {
        const r = {
            upload_type: "clear_user"
        };

        return [
            r,
        ];
    }

    pay(param: UploadPayParam): Json[] {
        const r = {
            upload_type: "charge",
            amount: param.amount,
            order_id: param.orderID,
            product: param.goodsName,
            currency_type: param.currencyType,
            payment: param.payment,
        };

        return [
            r as any as Json,
        ]
    }

    custom(type: string, value?: Json): Json {
        let r: ITabDBEvent = null;

        if (value) {
            switch (type) {
                case "event":
                    if (typeof value["event_code"] === typeof "") {
                        let p: ITabDBEvent = {
                            upload_type: "event",
                            event_code: value["event_code"],
                            properties: value["properties"] || "",
                        };
                        r = p;
                    }
                    break;
                default:
                    break;
            }
        }
        return r as any as Json;
    }

    private createDefaultUser(): IUploadUser {
        return {
            serverID: null,
            serverName: null,
            userID: null,
            userName: null,
            userType: null,
            userSex: null,
            userAge: -1,
            roleLevel: -1,
            roleVipLevel: -1,
            roleBalance: null,
            partyName: null,
            extend: null,
        }
    }
}

// TabDB: 自定义事件
interface ITabDBEvent {
    upload_type: "event",
    event_code: string,     // 在控制台中配置得到的事件编码
    properties?: string     // 事件属性.需要和控制台的配置匹配
}