
import { Json } from "pi_tools/type";


/**
 * 支付参数
 */
export interface UploadPayParam {

    /**
     * @note 某些平台 的 支付总金额 最小单位是 元 或 角，请注意了解下对应平台的 支付单位 ！
     */
    amount: number,          // （必填）总金额；单位：分；

    orderID: string,         // （必填）订单ID；
    goodsName: string,       // （必填）商品名； 

    currencyType?: CurrencyType, // 支付货币类型；默认值："CNY"

    /**
     * 支付宝 微信 QQ小游戏 必填
     */
    orderInfo?: any;     // 订单内容

    /**
     * @note Apple 支付，必须填 商品ID；
     */
    goodsID?: string,        // 商品ID；默认值：""

    /**
     * @note goodsCount 必须 整除 amount
     */
    goodsCount?: number,    // 商品数量；默认值：1
    goodsDesc?: string,     // 商品描述；默认值：""
    extendParams?: string,  // 额外参数；默认值：""
    goodsurl?: string; // 物品的图片url

    /**
     * @note 动玩 必填
     * 
     * 限制 数量 = 兑换比例 * 金额（元）
     */
    ratio?: number,         // 兑换比例（固定值） 1元能获得得数量； 默认值：1
    item_name?: string,     // 游戏币单位名称（固定值） 例如：元宝 金币 钻石； 默认值：""

    /**
     * @note 我爱游 必填
     * @note Ohayoo 必填
     */
    notifyUrl?: string;     // 支付回调URL

    /**
     * @note 我爱游 必填
     */
    account?: string;       // 渠道方登录时返回的account

    /**
     * @note 华为 必填
     */
    country?: string;        // 国家/地区码 必须符合ISO 3166标准。默认值："CN"
    price_type?: number;     // 商品类型：0消耗性商品；1非消耗性商品  默认值：0

    /**
     * @note 微信小游戏 QQ游戏大厅 必填
     */
    zoneId?: string,  // 游戏分区id
    offerId?: string;        // 米大师平台分配的应用id
    env?: number;     // 环境配置 0：米大师正式 1：米大师测试  默认值：0

    /**
     * @note google 必填
     */
    productType?: ProductType;  // 商品类型 inapp: 应用内商品  subs: 订阅

    /**
     * @note 华为快游戏 必填
     */
    merchantName?: string;  // 商户名称

    /**
     * @note 起点阅文游戏 4399
     */
    serverid?: number;       // 游戏区服id
    itemid?: number;        //  游戏道具或者金币等id (如果未在运营同学处配置充值档, 则不需要传该字段)

    /**
     * @note 微信客服支付 必填
     */
    appid?: string;   // 微信小游戏的APPID
    sendMessageTitle?: string; // 会话内消息卡片标题
    sendMessageImg?: string; // 会话内消息卡片图片路径

    /**
     * @note quick海外 上报
     */
    eventToken?: string; // 上报事件token

    /**
     * 所有渠道 必填
     */
    payment?: UploadPayType,       // 比如: "zfb", "wx", "yc", "my", "quicksdk", "ohy_sdk"
}

/**
 * 支付 结果 信息
 */
export interface UploadPayInfo {

}

/**
 * 货币 类型
 */
export enum CurrencyType {
    CNY = "CNY", // 人民币 
    USD = "USD", // 美元
    EUR = "EUR", // 欧元 
}

/**
 * 支付 类型
 * Android/iOS 底层的名字
 */
export enum UploadPayType {
    wx = 'wx',
    wx2 = 'wx',
    wx3 = 'wx',
    zfb = 'zfb',
    apple = 'apple',
    wx_web = 'wx_web',
    zfb_web = 'zfb_web',

    yuchen = 'yc', // 煜辰
    mengyou = 'my', // 萌友
    dongwan = 'dw', // 动玩
    woaiyou = 'way',  // 我爱游
    huawei = 'huawei', // 华为
    steam = 'steam', // steam
    wegame = 'wegame', // wegame
    quicksdk = 'quicksdk', // quicksdk
    ohy_sdk = 'ohy_sdk', // ohy_sdk
    google = 'google',  // google
    wx_minigame = 'wx_minigame', // 微信小游戏
    qq_minigame = 'qq_minigame', // QQ小游戏
    qq_game = 'qq_game', // QQ游戏大厅
    qq_zone = 'qq_zone',  // QQ空间
    qq_game_pc = 'qq_game_pc', // QQ游戏大厅 pc
    muxue = 'muxue', // 暮雪
    hw_minigame = 'hw_minigame', // 华为快游戏
    wx_customer_service = 'wx_customer_service', // 微信客服
    quicksdkio = 'quicksdkio', // quick海外
    quicksdkio2 = 'quicksdkio', // quick海外new
    quicksdkios = 'qkios',                  // quick ios

    dy_minigame = 'dy_minigame',  // 抖音小游戏
    qingcheng = 'qingcheng', // 倾程
    qingcheng_vivo = 'qingcheng_vivo', // 倾程 vivo
    qingcheng_zfb = 'qingcheng_zfb', // 倾程 支付宝
    qingcheng_mt = 'qingcheng_mt', // 倾程 美团
    quick_h5 = 'quick_h5',   // quick h5
    sanxiang = 'sanxiang',   // 弎湘
    qidian = "qidian",       // 起点阅文游戏
    h5_4399 = "4399",        // 4399 h5页游
    pisdk = 'pisdk',        // 非聚合SDK标识 可以多种支付方式共存 或无需与底层通信的方式
    ronghe = 'ronghe',      // 大千融合
    daqian = 'daqian',       // 大千sdk
    tapdb = 'tapdb',           //tapSDK
    tanwan = 'tanwan'          //贪玩
}

/**
 * 支付渠道名
 * 服务器端
 */
export enum UploadPayChannelName {
    wx = 'wx_app', // 微信APP
    wx2 = 'wx_app2', // 微信APP
    wx3 = 'wx_app3', // 微信APP
    wx_web = 'wx_wap', // 微信网页
    wx_minigame = 'wx_minigame', // 微信小游戏
    qq_minigame = 'qq_minigame', // QQ小游戏
    qq_game = 'qq_game',  // QQ游戏大厅
    qq_zone = 'qq_zone',  // QQ空间
    qq_game_pc = 'qq_game_pc', // QQ游戏大厅 pc
    qq_dt = 'qq_dt',           // qq大厅(新加)
    apple = 'apple_pay',  // 苹果支付

    zfb = 'alipay_app',  // 支付宝APP
    zfb_web = 'alipay_wap',  // 支付宝网页

    yuchen = 'youxi727', // 煜辰
    mengyou = 'mengyou', // 萌友
    dongwan = 'xiandou', // 动玩
    woaiyou = 'aiwan',  // 我爱游
    huawei = 'huawei', // 华为
    steam = 'steam_pc', // steam
    wegame = 'wegame_pc', // wegame
    quicksdk = 'quicksdk', // quicksdk
    ohy_sdk = 'ohayoo',    // ohayoo
    google = 'google',  // google
    muxue = 'muxue', // 暮雪
    hw_minigame = 'hwminigame', // 华为快游戏
    quick_h5 = 'quicksdk',   // quick h5
    sanxiang = 'sanxiang',   // 弎湘
    qidian = "qidian",      // 起点阅文游戏
    h5_4399 = "4399",       // 4399 h5页游
    wx_customer_service = 'wx_customer_service', // 微信客服
    quicksdkio = 'quicksdk_oversea', // quick海外
    quicksdkio2 = 'quicksdk_google', // quick海外new
    quicksdkios = 'quicksdk_oversea', // quick ios
    qingcheng = 'qingcheng',       // 倾程
    qingcheng_vivo = 'qingcheng',  // 倾程 vivo
    qingcheng_zfb = 'qingcheng',   // 倾程 支付宝
    qingcheng_mt = 'qingcheng',    // 倾程 美团
    dy_minigame = 'dy_minigame',   // 抖音小游戏
    ronghe = 'ronghe',             // 大千融合
    daqian = 'ronghe',              // 大千sdk
    tapdb = 'tapdb',                 //tap支付
    tanwan = 'tanwan'               //贪玩
}

/**
 * google 商品类型
 */
export enum ProductType {
    inapp = 'inapp',  // 应用内商品
    subs = 'subs' // 订阅
}

/**
 * 下单参数
 * 
 */
export interface IUnifiedOrder {
    app: string,  // 支付平台分配的APP名
    channel: UploadPayChannelName,  // 渠道名
    amount: number,  // 金额 单位：分
    currency: string, // 货币类型
    subject?: string,  // 订单标题
    body?: string,  // 商品描述
    extra?: string,  // 额外信息 部分渠道特殊要求
    note?: string,  // 备注 预留字段
    ratio?: number,  // 兑换比例（固定值） 1元能获得得数量； 默认值：1

    /**
     * steam 支付 需要
     */
    usersession?: string,  // client 或 web  默认值：client
    itemcount?: number, // 商品数量 默认值：1
    language?: string,  // 语言 默认值：zh-CN

    /**
     * 微信小游戏 qq游戏大厅 支付 需要
     */
    zoneId?: string,  // 分区id
    env?: number, // 环境配置 0：米大师正式 1：米大师测试  默认值：0

    /**
     * 弎湘 支付 需要
     */
    roleId?: string,  // 游戏玩家角色id

    /**
     * qq小游戏 QQ空间 支付 需要
     */
    goodid?: string,  // 在开发者管理端上架道具的id
    good_num?: number, // 商品数量 默认值：1
    goodsurl?: string; // 物品的图片url

    /**
     * 倾程 支付 需要
     */
    cp_server_id?: string, // CP区服id
    cp_server_name?: string, // CP区服名称
    cp_role_id?: string, // CP角色id
    cp_role_name?: string, // CP角色名称
}

/**
 * 下单 返回值
 */
export interface IOrderInfo {
    pt_oid: string,
    credential: string
}

// ============================ 给 实现用 的 接口

/**
 * 平台实现层 使用
 */
export interface IPlatformPay {

    /**
     * 将 UploadPayParam 转换成 特定平台的 参数
     */
    convertParameter(param: UploadPayParam): Json;

    /**
     * 将 特定平台的 返回值 转换成 IPayInfo
     */
    convertInfo(info: Json): UploadPayInfo;
}
